const klay = require('klayjs');
const graphUtil = require('./utils.js');

const DEFAULT_OPTIONS = {
  marginLeft: 20,
  marginTop: 20
};

/* Convert platformsh json to a kgraph */
function convert(platformshTopology = [], nodeSize = 20) {
  const nodes = {
    ...platformshTopology.webapps,
    ...platformshTopology.services,
    ...platformshTopology.workers
  };
  const nodesArray = Object.keys(nodes);

  const children = nodesArray.map(nodeName => ({
    id: nodeName,
    width: nodeSize,
    height: nodeSize,
    type: nodes[nodeName].type,
    data: {...nodes[nodeName], name: nodeName},
    properties: {
      nodeLabelPlacement: 'OUTSIDE V_TOP H_LEFT',
      portAlignment: 'CENTER',
      portSpacing: 1,
      portSide: 'NORTH',
      nodePlace: 'LINEAR_SEGMENTS',
      alignment: 'CENTER'
    },
    labels: [ { text: nodeName } ]
  }));

  const edges = nodesArray
    .reduce((links, nodeName) => {
      const node = nodes[nodeName];

      return links.concat(Object.keys(node.relationships).map(relationshipName => ({
        id: `${nodeName}-link-${relationshipName}-link`,
        source: nodeName,
        target: node.relationships[relationshipName].split(':')[0]
      })));
    }, []);

  const kGraph = {
    id: 'root',
    properties: {
      direction: 'DOWN',
      spacing: 40
    },
    children,
    edges
  };

  return kGraph;
}

/* Generate a layered graph*/
function generateLayout(graph, success) {
  klay.layout({
    graph,
    success
  });
}

function drawGraph(layout, options = DEFAULT_OPTIONS) {
  var raphael = require('raphael');

  raphael.setWindow(window);

  const paper = raphael(
    document.getElementById('service-tree'),
    layout.width + options.marginLeft,
    layout.height + options.marginTop
  );

  paper.canvas.setAttribute('id', 'serviceTree');

  layout.children.map(node => {
    graphUtil.drawNode(
      paper,
      node.x + options.marginLeft,
      node.y + options.marginTop,
      node.type === 'app' ? 'python' : 'mongodb',
      node.data, options.nodeSize
    );
    const textNode = paper.text(
      node.x + options.marginLeft + node.labels[0].x + node.width,
      node.y + options.marginTop + node.labels[0].y - node.height,
      node.labels[0].text
    );

    layout.edges.map(link => {
      const sourcePoint = {x: link.sourcePoint.x + options.marginLeft, y: link.sourcePoint.y + options.marginTop};
      const targetPoint = {
        x: link.targetPoint.x + options.marginLeft,
        y: link.targetPoint.y + (options.marginTop - options.nodeSize)
      };
      const bendPoints = link.bendPoints && link.bendPoints.map(bp => ({
        x: bp.x + options.marginLeft,
        y: bp.y + options.marginTop
      }));

      return graphUtil.drawLink(paper, link.id, sourcePoint, targetPoint, bendPoints, options.nodeSize);
    });

    textNode.node.setAttribute('id', `${node.name}-text`);
  });
}

export default (platformshTopology, nodeSize = 20) => new Promise((resolve, reject) => {
  const kGraph = convert(platformshTopology, nodeSize);

  generateLayout(kGraph, function (layout) {
    const svg = drawGraph(layout, {nodeSize, ...DEFAULT_OPTIONS});

    resolve({
      svg,
      layout
    });
  });
});
